# This file is generated by pto-pre-build.sh. Do not edit it manually.
# Mon 24 Jun 2024 04:06:51 AM EDT
# SOURCE NAME: /b/Dropbox/jgt/drop/fnml.py
# ------------------------------------
import argparse
import subprocess

def tide(instrument, timeframe, buysell):
  subprocess.run(['tide', instrument, timeframe, buysell])

def pds(instrument, timeframe):
  subprocess.run(['jgtfxcli', '-i', instrument, '-t', timeframe, '--full'])

def cds(instrument, timeframe, use_fresh=False):
  old_or_fresh = '-old' if not use_fresh else '--fresh'
  subprocess.run(['jgtcli', '-i', instrument, '-t', timeframe, '--full', '-mfi', '-ba', '-ta', old_or_fresh])

def ocds(instrument, timeframe):
  subprocess.run(['jgtcli', '-i', instrument, '-t', timeframe, '--full', '-mfi', '-ba', '-ta', '-old'])

def ttf(instrument, timeframe):
  subprocess.run(['ptojgtmlttfprotocli', '-i', instrument, '-t', timeframe, '--full', '-fr'])

def mx(instrument, timeframe, use_fresh=False):
  old_or_fresh = '-old' if not use_fresh else '--fresh'
  subprocess.run(['jgtmlcli', '-i', instrument, '-t', timeframe, '-ba', '-ta', old_or_fresh])

def ttfmxwf(instrument, use_fresh=False):
  for t in ["M1", "W1", "D1", "H4"]:
    print(f"Processing {instrument} for timeframe {t}")
    print("  CDS....")
    cds(instrument, t, use_fresh)
    if t != "M1":
      print("  TTF....")
      ttf(instrument, t)
    if t != "M1" and t != "W1":
      print("  MX....")
      mx(instrument, t)
      
def ttfwf(instrument, use_fresh=False):
  for t in ["M1", "W1", "D1", "H4"]:
    print(f"Processing {instrument} for timeframe {t}")
    print("  CDS....")
    cds(instrument, t, use_fresh)
    if t != "M1":
      print("  TTF....")
      ttf(instrument, t)
        
def main():
  parser = argparse.ArgumentParser(description="CLI equivalent of bash functions")
  subparsers = parser.add_subparsers(dest='command')

  parser_tidealligator = subparsers.add_parser('tide', help='Run the pto tidealligator')
  parser_tidealligator.add_argument('-i','--instrument', help='Instrument')
  parser_tidealligator.add_argument('-t','--timeframe', help='Timeframe')
  parser_tidealligator.add_argument('buysell', help='Buy or Sell')

  parser_prep_pds_01 = subparsers.add_parser('pds', help='Refresh the PDS full for an instrument and timeframe')
  parser_prep_pds_01.add_argument('-i','--instrument', help='Instrument')
  parser_prep_pds_01.add_argument('-t','--timeframe', help='Timeframe')

  parser_prep_cds_05 = subparsers.add_parser('cds', help='Refresh the CDS')
  parser_prep_cds_05.add_argument('-i','--instrument', help='Instrument')
  parser_prep_cds_05.add_argument('-t','--timeframe', help='Timeframe')
  #--fresh flag to use the fresh data
  parser_prep_cds_05.add_argument('-new','--fresh', action='store_true', help='Use the fresh data')

  parser_prep_cds_06_old = subparsers.add_parser('ocds', help='Refresh the CDS from old PDS')
  parser_prep_cds_06_old.add_argument('-i','--instrument', help='Instrument')
  parser_prep_cds_06_old.add_argument('-t','--timeframe', help='Timeframe')
  
  
  parser_prep_ttf_10 = subparsers.add_parser('ttf', help='Refresh the TTF for an instrument and timeframe')
  parser_prep_ttf_10.add_argument('-i','--instrument', help='Instrument symbol')
  parser_prep_ttf_10.add_argument('-t','--timeframe', help='Timeframe')
  
  #ttfmxwf
  parser_post_ttfmxwf_14 = subparsers.add_parser('ttfmxwf', help='Refresh the TTF, MX and CDS for an instrument')
  parser_post_ttfmxwf_14.add_argument('-i','--instrument', help='Instrument symbol')
  parser_post_ttfmxwf_14.add_argument('-new','--fresh', action='store_true', help='Use the fresh data')

  parser_post_mx_15 = subparsers.add_parser('mx', help='Refresh the MX (using the TTF) for an instrument and timeframe')
  parser_post_mx_15.add_argument('-i','--instrument', help='Instrument symbol')
  parser_post_mx_15.add_argument('-t','--timeframe', help='Timeframe')

  parser_wf_ttf_prep_19 = subparsers.add_parser('ttfwf', help='Refresh TTF preparation for an instrument')
  parser_wf_ttf_prep_19.add_argument('-i','--instrument', help='Instrument symbol')
  parser_wf_ttf_prep_19.add_argument('-new','--fresh', action='store_true', help='Use the fresh data')

  args = parser.parse_args()
  #if no arguments are passed, print help
  if not vars(args).get('command'):
    parser.print_help()
    parser.exit()

  if args.command == 'tide':
    tide(args.instrument, args.timeframe, args.buysell)
  elif args.command == 'pds':
    pds(args.instrument, args.timeframe)
  elif args.command == 'cds':
    cds(args.instrument, args.timeframe, args.fresh)
  elif args.command == 'ocds':
    ocds(args.instrument, args.timeframe)
  elif args.command == 'ttf':
    ttf(args.instrument, args.timeframe)
  elif args.command == 'ttfmxwf':
    ttfmxwf(args.instrument, args.fresh)
  elif args.command == 'mx':
    mx(args.instrument, args.timeframe)
  elif args.command == 'ttfwf':
    ttfwf(args.instrument, args.fresh)

if __name__ == "__main__":
  main()


# This file is generated by pto-pre-build.sh. Do not edit it manually.
# Mon 24 Jun 2024 04:06:51 AM EDT
# SOURCE NAME: jgtml_obsds_240515_TIDE_SIGNALS.py
# ------------------------------------
#%% Imports 

from sklearn.linear_model import LinearRegression
from sklearn.metrics import mean_squared_error

import pandas as pd

CLI_DESCRIPTION = 'Process financial market data to analyze the performance of SELL signals generated by the "Fractal Divergent Bar" indicator within the "Tide Alligator" technical analysis tool.'

EPILOG="""
This Python script is designed to analyze the effectiveness of signals in the context of the "Tide Alligator" technical analysis tool, specifically those generated by the "Fractal Divergent Bar" indicator. The script reads historical market data for a specified instrument and timeframe, calculates technical indicators, and identifies valid SELL and BUY signals based on predefined criteria. 

The script then evaluates the profitability of each valid signal's direction, analyzes the distribution of profit potential across different signal types, and compares their performance. The results of the analysis are outputted as a CSV file, and a markdown file summarizing the key findings is also generated.
"""
X_ABSTRACT="""
# Abstract"""+EPILOG+"""



The script leverages the `jgtpy` and `jgtml` Python packages for data analysis and machine learning. It is an early prototype made during my coding mastery and is somehow customizable, allowing for the analysis of different instruments, timeframes, and signal types using the CLI. This script is part of a broader research project exploring the use of technical indicators in trading.

Future work includes further data analysis to identify additional patterns and insights, the development of a trading strategy based on the analysis findings. This script serves as a experiment to find valuable tool for improving trading strategies and decision-making by providing insights into the effectiveness of fractal divergent signals generated by the "Fractal Divergent Bar" indicator and their relationship with the "Tide Alligator" tool.

The 6 types of signals analyzed in this script are:

- all_signals
- sig_nmopen
- sig_basic_filtering
- sig_is_in_tteeth
- sig_tmopen_in_tteeth
- sig_tmopen_in_tlips

The script outputs the following metrics for each signal type:

- per_trade: Average profit per trade
- nb_entry: Number of valid signals for this type (could compare to the total number if signal of the "all_signals" column to learn how many signals are validated by this type of signal)
- tsum: Total profit across all signals
- title: Signal type

The script also generates a markdown file summarizing the key findings and a CSV file containing the detailed results of the analysis.

Details of each type of signals

- all_signals: All valid signals made by the jgtml.jtc.pto_target_calculation function
- sig_nmopen: Signals where the mouth of the "Regular Alligator" is open
- sig_basic_filtering: Signals where the price bar is out of the lips and teeth of the "Regular Alligator"
- sig_is_in_tteeth: Signals where the price bar has came back in the big teeth of the "Big Alligator" without the mouth being open or not.
- sig_tmopen_in_tteeth: Signals where the price bar has came back in the big teeth and the mouth of the "Big Alligator" is open (Exploring a Strategic entry when the mouth is open and we want to enter at the end of a retracement.  This explores signals that pulled back in the big teeth(the middest of the big balancing lines system))
- sig_tmopen_in_tlips: Signals where the price bar has came back in the big lips and the mouth of the "Big Alligator" is open (Exploring a Strategic entry when the mouth is open and we want to enter at the end of a retracement. This explores signals that pulled back in the big lips (the smallest of the big balancing lines system) )

Further analysis and interpretation of the results are needed to draw meaningful conclusions and insights from the data. The script is intended to be a starting point for more in-depth research and exploration of technical indicators in trading.

"""

import os

import numpy as np
import pandas as pd
try:
    from jgtpy import JGTCDS as cds
except:
    print("pip install -U jgtpy")

from jgtml import jtc

# Use jgtconstants column names from jgtutils
from jgtutils.jgtconstants import LOW,HIGH,FDBB,FDBS,BJAW,BLIPS,BTEETH,JAW,TEETH,LIPS,FDB_TARGET,VECTOR_AO_FDBS_COUNT,VECTOR_AO_FDBB_COUNT,VECTOR_AO_FDB_COUNT,TJAW,TLIPS,TTEETH
#FDB_TARGET=FDB_TARGET

def crop_dataframe(df, crop_last_dt: str = None, crop_start_dt: str = None):
    if crop_last_dt is not None:
        df = df[df.index <= crop_last_dt]
    if crop_start_dt is not None:
        df = df[df.index >= crop_start_dt]
    return df

#%% Contexts 
# Contexts

i = 'GBP/USD'
i = 'SPX500'
t = "H4"
t = "D1"

i=os.getenv("i",i)
t=os.getenv("t",t)

# FLAGS for the PROCESS we might want to configure or read from ENV
force_regenerate_mxfiles=False if os.getenv("force_regenerate_mxfiles") == "False" or os.getenv("force_regenerate_mxfiles") == "false" or os.getenv("force_regenerate_mxfiles") == "0" else True
mfi_flag=True
balligator_flag=True
talligator_flag=True
regenerate_cds=False if os.getenv("regenerate_cds") == "False" or os.getenv("regenerate_cds") == "false" or os.getenv("regenerate_cds") == "0" else True
use_fresh=False if os.getenv("use_fresh") == "False" or os.getenv("use_fresh") == "false" or os.getenv("use_fresh") == "0" else True
# use_fresh=True if os.getenv("use_fresh") == "True" or os.getenv("use_fresh") == "true" or os.getenv("use_fresh") == "1" else False
quiet=False if os.getenv("quiet") == "False" or os.getenv("quiet") == "false" or os.getenv("quiet") == "0" else True

# OUTPUTS Files
jgtdroot_default="/b/Dropbox/jgt" #$jgtdroot
jgtdroot=os.getenv("jgtdroot",jgtdroot_default)
#result_drop_base_default="/b/Dropbox/jgt/drop" #$jgtdroot
drop_subdir = "drop"
result_file_basename_default = os.getenv("result_file_basename_default") if os.getenv("result_file_basename_default") is not None else "jgtml_obsds_240515_TIDE_SIGNALS.result"
result_file_basename=result_file_basename_default

# Columns to select
sel_columns_base=[HIGH,LOW,JAW,TEETH,LIPS]

sel_columns_tide_alligator=[TJAW,TTEETH,TLIPS]
sel_columns_big_alligator=[BJAW,BTEETH,BLIPS]

# Select columns according to Flags (big alligator and tide alligator)
sel_columns_common=sel_columns_base + \
    (sel_columns_big_alligator if balligator_flag else None) +  \
        (sel_columns_tide_alligator if talligator_flag else None) + \
            [FDB_TARGET,VECTOR_AO_FDB_COUNT]

sel_columns_sell = sel_columns_common + [FDBS, VECTOR_AO_FDBS_COUNT]
sel_columns_buy = sel_columns_common + [FDBB, VECTOR_AO_FDBB_COUNT]

# print(sel_columns_common)
# print(sel_columns_sell)
# print(sel_columns_buy)
#%%

# sel_columns_sell = [HIGH,LOW,BJAW,BLIPS,BTEETH,TJAW,TTEETH,TLIPS,JAW,TEETH,LIPS,FDBS,FDB_TARGET,VECTOR_AO_FDBS_COUNT,VECTOR_AO_FDB_COUNT]
# sel_columns_buy = [HIGH,LOW,BJAW,BLIPS,BTEETH,TJAW,TTEETH,TLIPS,JAW,TEETH,LIPS,FDBB,FDB_TARGET,VECTOR_AO_FDBB_COUNT,VECTOR_AO_FDB_COUNT]

jgtpy_data_full_var_name = "JGTPY_DATA_FULL"

bs="S" # This prototype is for SELL signals only, we can extend it to BUY signals later
bs=os.getenv("bs",bs)

result_drop_base_override=None
source_dataset_archival_path_override=None
data_dir_override=None
archive_used_dataset=True
use_ttf_default = True


def main():
    global i,t,bs,print_abstract,regenerate_cds,result_drop_base_override,source_dataset_archival_path_override,quiet,data_dir_override,use_fresh
    import argparse
    parser = argparse.ArgumentParser(description=CLI_DESCRIPTION,epilog=EPILOG)
    parser.add_argument('-i','--instrument', type=str, help='Instrument')
    parser.add_argument('-t','--timeframe', type=str, help='Timeframe')
    #bs
    parser.add_argument('-bs','--buysell', type=str, default='S', help='Buy or Sell')
    #regenerate_cds
    parser.add_argument('-nocds','--dont_regenerate_cds', help='Dont Regenerate CDS',action='store_true')
    #use_fresh
    parser.add_argument('-nf','--nofresh', help='Dont Use Fresh (Assuming you need CDS Already generated and available)',action='store_true')
    #parser.add_argument('-old','--not_fresh', help='Dont Use Fresh (Assuming you need CDS Already generated and available)',action='store_true')

    #data_dir_override
    parser.add_argument('-dd','--data_dir_override', type=str, default=None, help='Data Directory Override path (JGTPY_DATA_FULL env var will be used if not provided)')
    #result_drop_base_override
    parser.add_argument('-rdb','--result_drop_base_override', type=str, default=None, help='Result Drop Base Override')
    #source_dataset_archival_path_override
    parser.add_argument('-sda','--source_dataset_archival_path_override', type=str, default=None, help='Source Dataset Archival Path Override')
    # quiet
    parser.add_argument('-q','--quiet', help='Quiet',action='store_true')
    #print X_ABSTRACT
    parser.add_argument('-abstract','--print_abstract', action='store_true', help='Print Abstract')
    parser.add_argument('-ttf','--use_ttf_rather_than_cds', action='store_true', help='Force usage of TTF as source of data (CDS + HTF data)')
    #result_file_basename
    parser.add_argument('-rfb','--result_file_basename', type=str, default=result_file_basename_default, help='Result File Base Name')

    args = parser.parse_args()
    print_abstract = args.print_abstract
    if print_abstract:
        print(X_ABSTRACT,flush=True)
        exit(0)

    print(f"i:{i} t:{t} bs:{bs}")

    i = args.instrument
    t = args.timeframe
    bs = args.buysell
    regenerate_cds = not args.dont_regenerate_cds #default is to regenerate
    result_drop_base_override = args.result_drop_base_override
    source_dataset_archival_path_override = args.source_dataset_archival_path_override
    quiet = args.quiet
    data_dir_override = args.data_dir_override
    use_fresh= False if args.nofresh else True
    
    #@STCIssue We would do that by default
    use_ttf_rather_than_cds=args.use_ttf_rather_than_cds if args.use_ttf_rather_than_cds else False #@STCGoal Integration of all Signal Evaluation in one Script - We want HTF(ttf) data to be used in the evaluation so we can have a better understanding of the context of the signal with higher timeframes (Zone,MFI, etc)


NB_CONTEXT_RUN=False
def _chg_logics_for_notebook_context():
    global use_fresh, regenerate_cds
    #TODO Chg logics for notebook context
    regenerate_cds=False
    use_fresh=False

import IPython

if IPython.get_ipython() is not None:
    # Code is running in a Jupyter notebook
    print("Notebook context detected")
    NB_CONTEXT_RUN=True
else:
    # Code is running outside a Jupyter notebook
    print("Notebook context not detected")
    NB_CONTEXT_RUN=False

if not NB_CONTEXT_RUN:
    if __name__ == "__main__":
        main()
else:
    _chg_logics_for_notebook_context()

#%% 
print(f"i:{i} t:{t} bs:{bs}")
#%%

# Where to save the results
result_drop_base=os.path.join(jgtdroot, drop_subdir) if result_drop_base_override is None else result_drop_base_override

if not os.path.exists(result_drop_base):
    result_drop_base=os.path.join(".", drop_subdir)
    os.makedirs(result_drop_base,exist_ok=True)
    

#Where source dataset is archived 
source_dataset_archival_path= os.path.join(result_drop_base, "data", "arch", "jgtml_240516") if source_dataset_archival_path_override is None else source_dataset_archival_path_override

# Columns to select based on the direction    
if bs=="S" or bs=="SELL" or bs=="sell":
    sel_columns = sel_columns_sell
else:
    if bs=="B" or bs=="BUY" or bs=="buy":
        sel_columns = sel_columns_buy

result_file_base = os.path.join(result_drop_base, result_file_basename)

result_file_md=result_file_base + ".md"
result_file_csv=result_file_base + ".csv"


try:
    os.makedirs(source_dataset_archival_path,exist_ok=True)
except:
    #exit the app with error about the directory
    print(f"Error creating directory {source_dataset_archival_path}")
    exit(1)



ifn=i.replace("/","-")
data_dir = os.getenv(jgtpy_data_full_var_name) if data_dir_override is None else data_dir_override
#idsfilepath = os.path.join(data_dir, "targets", "mx", f"{ifn}_{t}.csv")
#df = pd.read_csv(idsfilepath)
df=None
try:
    if not force_regenerate_mxfiles:
        df = jtc.readMXFile(i,t)
except:
    pass

#%pip install -U jgtpy

#set DF to None if column 'mfi' is not present (force regeneration)
if df is not None and 'mfi' not in df.columns: #TMP to force regeneration if column mfi is not present
    df = None

if df is None:
    df, sel1, sel2 = jtc.pto_target_calculation(i,t,
                                                mfi_flag=mfi_flag,talligator_flag=talligator_flag,
                                                balligator_flag=balligator_flag,
                                                regenerate_cds=regenerate_cds,
                                                use_fresh=use_fresh,
                                                use_ttf=use_ttf_default)

#%%
if NB_CONTEXT_RUN:
    quiet=False if os.getenv("quiet") is None else quiet

if not quiet:
    l=len(df)
    print(df.tail(1))
    print(df.columns)
    print("len(df):",len(df))

#%% 
"""
'Volume', 'Open', HIGH, LOW, 'Close', 'Median', 'ao', 'ac', JAW,
       TEETH, LIPS, 'fh', 'fl', 'fh3', 'fl3', 'fh5', 'fl5', 'fh8', 'fl8',
       'fh13', 'fl13', 'fh21', 'fl21', 'fh34', 'fl34', 'fh55', 'fl55', 'fh89',
       'fl89', 'fdbb', 'fdbs', 'fdb', 'aoaz', 'aobz', 'zlc', 'zlcb', 'zlcs',
       'zcol', 'sz', 'bz', 'acs', 'acb', 'ss', 'sb', 'price_peak_above',
       'price_peak_bellow', 'ao_peak_above', 'ao_peak_bellow', 'tmax', 'tmin',
       'p', 'l', TARGET, 'vaos', 'vaob', 'vaosc', 'vaobc', 'vaoc'],
      
"""

# create a dataset with only the columns we need.  
dfo = df[sel_columns]


# %%
if not quiet:
    print(dfo.columns)

# %%
# Select row where target is not 0
dfo = dfo[dfo[FDB_TARGET] != 0] 
#dfoprofit = dfo[dfo[TARGET] > 0] 
# %%
if not quiet:
    print(dfo)


#%% Direction
direction = bs #Might use only one variable in the future to avoid confusion
# %%
dfo_context = dfo[sel_columns].copy()

#
fdb_context_colname = FDBS
if direction=="B":
    fdb_context_colname = FDBB
dfo_context = dfo_context[dfo_context[fdb_context_colname] != 0].copy()





# dfo = dfo[dfo[TARGET] != 0] #Where target is not 0
# %%
dfo_context.tail(40)
# %%
all_context_signal_count = len(dfo_context)

target_colname = FDB_TARGET
all_signalsnal_sum=dfo_context[target_colname].sum()

print("count:",all_context_signal_count," sum0:",all_signalsnal_sum)

# %%
#Remove invalid signal when column High < lips
#@STCGoal Valid Signals are bellow the lips and teeth 
if bs=="S":
    sig_basic_filtering = dfo_context[dfo_context[LOW] > dfo_context[LIPS]].copy()
    sig_basic_filtering = sig_basic_filtering[sig_basic_filtering[LOW] > sig_basic_filtering[TEETH]]

    #@STCGoal Valid Signals when mouth is open
    sig_nmopen = sig_basic_filtering[sig_basic_filtering[JAW] < sig_basic_filtering[TEETH]].copy()
    sig_nmopen = sig_nmopen[sig_nmopen[TEETH] < sig_nmopen[LIPS]]
    sig_nmopen = sig_nmopen[sig_nmopen[JAW] < sig_nmopen[LIPS]]

    #sig_is_in_tteeth
    sig_is_in_tteeth = sig_basic_filtering[
    sig_basic_filtering[LOW] > sig_basic_filtering[TTEETH]
    ].copy()

    #sig_tmopen_in_tteeth
    sig_tmopen_in_tteeth = sig_basic_filtering[
        sig_basic_filtering[LOW] > sig_basic_filtering[TTEETH]
        ].copy()

    sig_tmopen_in_tteeth = sig_tmopen_in_tteeth[  #@a The Big Mouth Is Open
        sig_tmopen_in_tteeth[TLIPS] < sig_tmopen_in_tteeth[TTEETH]
        ]
    sig_tmopen_in_tteeth = sig_tmopen_in_tteeth[  #@a The Big Mouth Is Open
        sig_tmopen_in_tteeth[TTEETH] < sig_tmopen_in_tteeth[TJAW]
        ]        

    #sig_tmopen_in_tlips
    sig_tmopen_in_tlips = sig_basic_filtering[
        sig_basic_filtering[LOW] < sig_basic_filtering[TLIPS]
        ].copy()

    # the BMouth is Openned
    sig_tmopen_in_tlips = sig_tmopen_in_tlips[
        sig_tmopen_in_tlips[TLIPS] < sig_tmopen_in_tlips[TTEETH]
        ]
    sig_tmopen_in_tlips = sig_tmopen_in_tlips[
        sig_tmopen_in_tlips[TTEETH] < sig_tmopen_in_tlips[TJAW]
        ]
    
    #
                    

else:
    sig_basic_filtering = dfo_context[dfo_context[HIGH] < dfo_context[LIPS]].copy()
    sig_basic_filtering = sig_basic_filtering[sig_basic_filtering[HIGH] < sig_basic_filtering[TEETH]]

    #sig_nmopen
    sig_nmopen = sig_basic_filtering[sig_basic_filtering[JAW] > sig_basic_filtering[TEETH]].copy()
    sig_nmopen = sig_nmopen[sig_nmopen[TEETH] > sig_nmopen[LIPS]]
    sig_nmopen = sig_nmopen[sig_nmopen[JAW] > sig_nmopen[LIPS]]

    #sig_is_in_tteeth
    sig_is_in_tteeth = sig_basic_filtering[
    sig_basic_filtering[HIGH] < sig_basic_filtering[TTEETH]
    ].copy()

    #
    #sig_tmopen_in_tteeth
    sig_tmopen_in_tteeth = sig_basic_filtering[
        sig_basic_filtering[HIGH] < sig_basic_filtering[TTEETH]
        ].copy()

    sig_tmopen_in_tteeth = sig_tmopen_in_tteeth[  #@a The Big Mouth Is Open
        sig_tmopen_in_tteeth[TLIPS] > sig_tmopen_in_tteeth[TTEETH]
        ]
    sig_tmopen_in_tteeth = sig_tmopen_in_tteeth[  #@a The Big Mouth Is Open
        sig_tmopen_in_tteeth[TTEETH] > sig_tmopen_in_tteeth[TJAW]
        ]      
    
    #
    #sig_tmopen_in_tlips
    sig_tmopen_in_tlips = sig_basic_filtering[
        sig_basic_filtering[HIGH] > sig_basic_filtering[TLIPS]
        ].copy()

    # the BMouth is Openned
    sig_tmopen_in_tlips = sig_tmopen_in_tlips[
        sig_tmopen_in_tlips[TLIPS] > sig_tmopen_in_tlips[TTEETH]
        ]
    sig_tmopen_in_tlips = sig_tmopen_in_tlips[
        sig_tmopen_in_tlips[TTEETH] > sig_tmopen_in_tlips[TJAW]
        ]
    
    #






# INDEPENDENT OF DIRECTIONS
sig_basic_filtering_count = len(sig_basic_filtering)
sig_basic_filtering_sum=sig_basic_filtering[FDB_TARGET].sum()

sig_nmopen_count = len(sig_nmopen)
sig_nmopen_sum=sig_nmopen[FDB_TARGET].sum()





if not quiet:
    print("sig_basic_filtering:",sig_basic_filtering_count," sum:",sig_basic_filtering_sum)


# %%

if not quiet:
    print(sig_basic_filtering.tail(40))
    print("count3:",sig_nmopen_count," sum3:",sig_nmopen_sum)

# %%

if not quiet:
    print(sig_nmopen.tail(40))


# %%
if not quiet:
    print("count (no validation):",all_context_signal_count," sum0:",all_signalsnal_sum)
    print("count_2 sig_basic_filtering:",sig_basic_filtering_count," sum2:",sig_basic_filtering_sum)
    print("count_sell3 sig_nmopen:",sig_nmopen_count," sum3:",sig_nmopen_sum)






# # %%
# dfosell_vaoc_min_bar.tail(20)
# %%

# %%

result_of_this_analysis = """
Best Validation : not in lips and teeth

NEXT STEP : 

* Larger Timeframe insights.
"""


#@STCGoal  Eval Big Mouth 
# %%  BIG ALLIGATOR SELL - Eval Low is Bellow Big Teeth



sig_is_in_tteeth_count = len(sig_is_in_tteeth)
sig_is_in_tteeth_sum=sig_is_in_tteeth[FDB_TARGET].sum()

# %%

if not quiet:
    print(sig_is_in_tteeth.tail(40))
    print("sig_is_in_tteeth:",sig_is_in_tteeth_count," sum:",sig_is_in_tteeth_sum)




# %%  BIG ALLIGATOR  - Eval Low is Above Big Lips and Big Mouth is Open OR RELATED tO BUY

                                       
sig_tmopen_in_tteeth_count = len(sig_tmopen_in_tteeth)
sig_tmopen_in_tteeth_sum=sig_tmopen_in_tteeth[FDB_TARGET].sum()

# %%

if not quiet:
    print(sig_tmopen_in_tteeth.tail(40))
    print("count_sell2s2 Low is Above Big Teeth and Big Mouth is Open:",sig_tmopen_in_tteeth_count," sum2s2:",sig_tmopen_in_tteeth_sum)


# %%  BIG ALLIGATOR - Eval Low is Above Big Lips and Big Mouth is Open OR RELATED tO BUY

                                                                    
sig_tmopen_in_tlips_count = len(sig_tmopen_in_tlips)
sig_tmopen_in_tlips_sum=sig_tmopen_in_tlips[FDB_TARGET].sum()

# %%
if not quiet:
    print(sig_tmopen_in_tlips.tail(40))
    print("count_sell2s2 Low is Above Big Lips and Big Mouth is Open:",sig_tmopen_in_tlips_count," sum2s2:",sig_tmopen_in_tlips_sum)





# %%


def write_to_result_csv(i,t,direction,nb_entry, tsum, title,_df=None):
    per_trade=round(tsum/nb_entry,2)
    if direction == "sell" or direction=="Sell" or direction=="SELL":direction="S"
    if direction == "buy" or direction=="Buy" or direction=="BUY":direction="B"
    
    #per_trade="per_trade"
    with open(result_file_csv, "a") as file_object:
        tsum_rounded = round(tsum,2)
        file_object.write(f"{i},{t},{direction},{per_trade},{nb_entry},{tsum_rounded},{title}\n")
    if _df is not None:
        save_df_archives(i, t, title, _df)

def save_df_archives(i, t, title, _df,quiet=True):
    ifn = i.replace("/","-")
    csv_fn = f"{source_dataset_archival_path}/{ifn}_{t}_{title}.csv"
    if not quiet:
        print("Writing CSV to file:",csv_fn)
    _df.to_csv(csv_fn)

#save the original df
if archive_used_dataset:
    save_df_archives(i, t, "original", df)
#write_to_result_csv("instrument","timeframe","nb_entry","tsum","title")

def write_to_result_md(entry_line):
    with open(result_file_md, "a") as file_object:
        file_object.write(f"{entry_line}\n")

def print_res(i,t,direction,nb_entry, tsum, title,_df=None):
    per_trade=round(tsum/nb_entry,2)
    tsum_rounded = round(tsum,2)
    entry_line = f"{i}_{t} {direction} pt:{per_trade} t:{nb_entry} sum:{tsum_rounded} title:{title}"
    print(entry_line)
    write_to_result_md(entry_line)
    write_to_result_csv(i,t,direction,nb_entry, tsum, title,_df)
# %%

write_to_result_md("  ")
write_to_result_md("----")
write_to_result_md("  ")
write_to_result_md("==============================================================")
print_res(i,t,direction,all_context_signal_count,all_signalsnal_sum,"all_signals",dfo_context)
print_res(i,t,direction,sig_nmopen_count,sig_nmopen_sum,"sig_nmopen",sig_nmopen)
write_to_result_md("==============================================================")
print_res(i,t,direction,sig_basic_filtering_count,sig_basic_filtering_sum,"sig_basic_filtering",sig_basic_filtering)
write_to_result_md("==============================================================")
print_res(i,t,direction,sig_is_in_tteeth_count,sig_is_in_tteeth_sum,"sig_is_in_tteeth_sum",sig_is_in_tteeth)
print_res(i,t,direction,sig_tmopen_in_tlips_count,sig_tmopen_in_tlips_sum,"sig_tmopen_in_tlips",sig_tmopen_in_tlips)
#pt:32.42 t:132 sum:4279.999999999999 title:count_sell2s2 Low is Above Big Lips and Big Mouth is Open
print_res(i,t,direction,sig_tmopen_in_tteeth_count,sig_tmopen_in_tteeth_sum,"sig_tmopen_in_tteeth_sum",sig_tmopen_in_tteeth)
write_to_result_md("==============================================================")


#@STCIssue SPX500 D1, sig_tmopen_in_tteeth_sum has an interesting value when the Signal bar is in the big mouth
#pt:83.2 t:99 sum:8237.0 title:sig_tmopen_in_tteeth_sum
# %%


# %% Imports
import pandas as pd
import numpy as np
import os
import sys

sys.path.insert(0, os.path.abspath(os.path.dirname(__file__)))
from jgtpy import JGTPDSP as pds
import tlid


from jgtutils.jgtos import get_data_path


from jgtutils.jgtconstants import *


# %% Functions


def _crop_dataframe(df, crop_last_dt: str = None, crop_start_dt: str = None):
    if crop_last_dt is not None:
        df = df[df.index <= crop_last_dt]
    if crop_start_dt is not None:
        df = df[df.index >= crop_start_dt]
    return df


def calculate_target_variable_min_max(
    dfsrc,
    crop_last_dt=None,
    crop_start_dt=None,
    WINDOW_MIN=1,
    WINDOW_MAX=150,
    set_index=True,
    rounder=4,
    pipsize=-1,
):
    df = dfsrc.copy()
    
    
    if crop_last_dt is not None or crop_start_dt is not None:
        df = _crop_dataframe(df, crop_last_dt, crop_start_dt)

    # reset index before we iterate
    try:
        df.reset_index(drop=False, inplace=True)
    except:
        pass

    # Initialize the tmax and tmin columns with NaN values
    df["tmax"] = float("nan")
    df["tmin"] = float("nan")
    df["p"] = float("nan")
    df["l"] = float("nan")
    df["target"] = float("nan")

    # Calculate the maximum and minimum Close value in the window range for each row
    for i in range(WINDOW_MIN, len(df) - WINDOW_MAX):
        # FDBS
        if df.loc[i, "fdbs"] == 1.0:
            df.loc[i, "tmax"] = df.loc[i : i + WINDOW_MAX, "Close"].min()
            df.loc[i, "tmin"] = df.loc[i : i + WINDOW_MAX, "Close"].max()

            if df.loc[i, "High"] < df.loc[i, "tmin"]:
                df.loc[i, "l"] = round(df.loc[i, "High"] - df.loc[i, "Low"], rounder)
                df.loc[i, "target"] = round(
                    -1 * (df.loc[i, "High"] - df.loc[i, "Low"]), rounder
                )
            else:
                df.loc[i, "p"] = round(df.loc[i, "Low"] - df.loc[i, "tmax"], rounder)
                df.loc[i, "target"] = round(
                    df.loc[i, "Low"] - df.loc[i, "tmax"], rounder
                )
        # FDBB
        if df.loc[i, "fdbb"] == 1.0:
            df.loc[i, "tmax"] = df.loc[i : i + WINDOW_MAX, "Close"].max()
            df.loc[i, "tmin"] = df.loc[i : i + WINDOW_MAX, "Close"].min()

            if df.loc[i, "Low"] > df.loc[i, "tmin"]:
                df.loc[i, "l"] = round(df.loc[i, "High"] - df.loc[i, "Low"], rounder)
                df.loc[i, "target"] = round(
                    -1 * (df.loc[i, "High"] - df.loc[i, "Low"]), rounder
                )
            else:
                df.loc[i, "p"] = round(df.loc[i, "tmax"] - df.loc[i, "High"], rounder)
                df.loc[i, "target"] = round(
                    df.loc[i, "tmax"] - df.loc[i, "High"], rounder
                )

    # Fill NaN with zero for columns tmax and tmin
    df["tmax"] = df["tmax"].fillna(0)
    df["tmin"] = df["tmin"].fillna(0)
    df["p"] = df["p"].fillna(0)
    df["l"] = df["l"].fillna(0)
    df["target"] = df["target"].fillna(0)
    # After calculating the 'target' column
    if pipsize != -1:
        df["target"] = df["target"] / pipsize

    # @q Maybe set backnthe index !??
    if set_index:
        #parse Date as datetime
        df['Date'] = pd.to_datetime(df['Date'])
        df.set_index("Date", inplace=True)
        
    return df


def pto_target_calculation(
    i:str,
    t:str,
    crop_start_dt=None,
    crop_end_dt=None,
    tlid_tag=None,
    WINDOW_MIN:int=1,
    WINDOW_MAX:int=150,
    output_report_dir=None,
    pto_vec_fdb_ao_vector_window_flag=True,
    drop_calc_col=True,
    write_reporting=True,
    calc_col_to_drop_names=["tmax", "tmin", "p", "l"],
    sel_1_suffix="_sel",
    sel_1_keeping_columns=["High","Low", "fdbs", "fdbb", "tmax", "tmin", "p", "l", "target"],
    sel_2_suffix="_tnd",
    sel_2_keeping_columns=["Open", "High", "Low", "Close", "fdbs", "fdbb", "target"],
    pto_vec_fdb_ao_out_s_name="vaos",
    pto_vec_fdb_ao_out_b_name="vaob",
    pto_vec_fdb_ao_in_s_sig_name="fdbs",
    pto_vec_fdb_ao_in_s_win_end_sig_name="zlcb",
    pto_vec_fdb_ao_in_b_sig_name="fdbb",
    pto_vec_fdb_ao_in_b_win_end_sig_name="zlcs",
    pto_vec_fdb_ao_in_t_val_name="ao",
    additional_columns_to_drop=None,
    selected_columns_to_keep=None,
    save_outputs=True,
    only_if_target_exist_n_not_zero=True,
    use_fresh=True,
    keep_bid_ask=True,
    regenerate_cds=False,
    gator_oscillator_flag=False,
    mfi_flag=True,
    balligator_flag=False,
    balligator_period_jaws=89,
    largest_fractal_period=89,
    talligator_flag=False,
    talligator_period_jaws=377,
    use_ttf=True,
):
    """
    Prototype Calculation of target based on the given POV parameters and output to file with report.

    Args:
        i (int): The value of i.get_fdb_ao_vector_window
        t (int): The value of t.
        crop_start_dt (datetime, optional): The start date for cropping. Defaults to None.
        crop_end_dt (datetime, optional): The end date for cropping. Defaults to None.
        tlid_tag (str, optional): The TLID tag. Defaults to None.
        output_report_dir (str, optional): The output report directory. Defaults to None.
        process_fdb_ao_vector_window (bool, optional): If True, process the fdb_ao_vector_window. Defaults to False.
        drop_calc_col (bool, optional): If True, drop the calculated columns. Defaults to False.
        write_reporting (bool, optional): If True, write the reporting. Defaults to True.
        calc_col_to_drop_names (list, optional): The list of column names to drop. Defaults to ["tmax", "tmin", "p", "l"].
        sel_1_suffix (str, optional): The suffix for the first selection. Defaults to "_sel".
        sel_1_keeping_columns (list, optional): The list of columns to keep for the first selection. Defaults to ["Low", "fdbs", "fdbb", "tmax", "tmin", "p", "l", "target"].
        sel_2_suffix (str, optional): The suffix for the second selection. Defaults to "_tnd".
        sel_2_keeping_columns (list, optional): The list of columns to keep for the second selection. Defaults to ["Open", "High", "Low", "Close", "fdbs", "fdbb", "target"].  
        pto_vec_fdb_ao_out_s_name (str, optional): The name of the output for fdb_ao_vector_window. Defaults to "vaos".
        pto_vec_fdb_ao_out_b_name (str, optional): The name of the output for fdb_ao_vector_window. Defaults to "vaob".
        pto_vec_fdb_ao_in_s_sig_name (str, optional): The name of the input for fdb_ao_vector_window. Defaults to "fdbs".
        pto_vec_fdb_ao_in_s_win_end_sig_name (str, optional): The name of the input for fdb_ao_vector_window. Defaults to "zlcb".
        pto_vec_fdb_ao_in_b_sig_name (str, optional): The name of the input for fdb_ao_vector_window. Defaults to "fdbb".
        pto_vec_fdb_ao_in_b_win_end_sig_name (str, optional): The name of the input for fdb_ao_vector_window. Defaults to "zlcs".
        pto_vec_fdb_ao_in_t_val_name (str, optional): The name of the input for fdb_ao_vector_window. Defaults to "ao".
        additional_columns_to_drop (list, optional): The list of additional columns to drop. Defaults to None.
        selected_columns_to_keep (list, optional): The list of selected columns to keep. Defaults to None.
        save_outputs (bool, optional): If True, save the outputs. Defaults to True.
        only_if_target_exist_n_not_zero (bool, optional): If True, only if target exists and not zero. Defaults to True.
        use_fresh (bool, optional): If True, use fresh data. Defaults to False.
        keep_bid_ask (bool, optional): If True, keep bid and ask. Defaults to True.
        regenerate_cds (bool, optional): If True, regenerate the CDS. Defaults to False.
        gator_oscillator_flag (bool, optional): If True, calculate the Gator Oscillator. Defaults to False.
        mfi_flag (bool, optional): If True, calculate the MFI. Defaults to False.
        balligator_flag (bool, optional): If True, calculate the Alligator. Defaults to False.
        balligator_period_jaws (int, optional): The period for the Alligator jaws. Defaults to 89.
        largest_fractal_period (int, optional): The period for the largest fractal. Defaults to 89.
        talligator_flag (bool, optional): If True, calculate the T-Alligator. Defaults to False.
        talligator_period_jaws (int, optional): The period for the T-Alligator jaws. Defaults to 377.

        

    Returns:
        df_result_tmx (pandas.DataFrame): The DataFrame containing the calculated PTO target.
        sel1 (pandas.DataFrame): The DataFrame containing the selected columns.
        sel2 (pandas.DataFrame): The DataFrame containing the selected columns.
    """
    if tlid_tag is None:
        tlid_tag = tlid.get_minutes()

    #default_jgtpy_data_full = "full/data"
    default_jgtpy_data_full = "/var/lib/jgt/full/data"
    data_dir_full = os.getenv("JGTPY_DATA_FULL", default_jgtpy_data_full)
    indir_cds = os.path.join(data_dir_full, "cds")
    outdir_tmx = os.path.join(
        data_dir_full, "targets", "mx"
    )  # @STCIssue Hardcoded path future JGTPY_DATA_FULL/.../mx
    #make the outdir_tmx if not exist
    os.makedirs(outdir_tmx, exist_ok=True)
    
    df_result_tmx, sel1, sel2 = _pov_target_calculation_n_output240223(
        indir_cds=indir_cds,
        outdir_tmx=outdir_tmx,
        crop_start_dt=crop_start_dt,
        crop_end_dt=crop_end_dt,
        i=i,
        t=t,
        tlid_tag=tlid_tag,
        WINDOW_MIN=WINDOW_MIN,
        WINDOW_MAX=WINDOW_MAX,
        output_report_dir=output_report_dir,
        pto_vec_fdb_ao_vector_window_flag=pto_vec_fdb_ao_vector_window_flag,
        drop_calc_col=drop_calc_col,
        calc_col_to_drop_names=calc_col_to_drop_names,
        sel_1_suffix=sel_1_suffix,
        sel_1_keeping_columns=sel_1_keeping_columns,
        sel_2_suffix=sel_2_suffix,
        sel_2_keeping_columns=sel_2_keeping_columns,
        pto_vec_fdb_ao_out_s_name=pto_vec_fdb_ao_out_s_name,
        pto_vec_fdb_ao_out_b_name=pto_vec_fdb_ao_out_b_name,
        pto_vec_fdb_ao_in_s_sig_name=pto_vec_fdb_ao_in_s_sig_name,
        pto_vec_fdb_ao_in_s_win_end_sig_name=pto_vec_fdb_ao_in_s_win_end_sig_name,
        pto_vec_fdb_ao_in_b_sig_name=pto_vec_fdb_ao_in_b_sig_name,
        pto_vec_fdb_ao_in_b_win_end_sig_name=pto_vec_fdb_ao_in_b_win_end_sig_name,
        pto_vec_fdb_ao_in_t_val_name=pto_vec_fdb_ao_in_t_val_name,
        additional_columns_to_drop=additional_columns_to_drop,
        selected_columns_to_keep=selected_columns_to_keep,
        save_outputs=save_outputs,
        write_reporting=write_reporting,
        only_if_target_exist_n_not_zero=only_if_target_exist_n_not_zero,
        use_fresh=use_fresh,
        keep_bid_ask=keep_bid_ask,
        regenerate_cds=regenerate_cds,
        gator_oscillator_flag=gator_oscillator_flag,
        mfi_flag=mfi_flag,
        balligator_flag=balligator_flag,
        balligator_period_jaws=balligator_period_jaws,
        largest_fractal_period=largest_fractal_period,
        talligator_flag=talligator_flag,
        talligator_period_jaws=talligator_period_jaws, 
        use_ttf=use_ttf
    )
    return df_result_tmx, sel1, sel2


def _pov_target_calculation_n_output240223(
    indir_cds,
    outdir_tmx,
    crop_start_dt,
    crop_end_dt,
    i,
    t,
    tlid_tag,
    WINDOW_MIN=1,
    WINDOW_MAX=150,
    output_report_dir=None,
    pto_vec_fdb_ao_vector_window_flag=True,
    drop_calc_col=True,
    write_reporting=True,
    calc_col_to_drop_names=["tmax", "tmin", "p", "l"],
    sel_1_suffix="_sel",
    sel_1_keeping_columns=[
        "High",
        "Low",
        "fdbs",
        "fdbb",
        "tmax",
        "tmin",
        "p",
        "l",
        "target",
    ],
    sel_2_suffix="_tnd",
    sel_2_keeping_columns=["Open", "High", "Low", "Close", "fdbs", "fdbb", "target"],
    pto_vec_fdb_ao_out_s_name="vaos",
    pto_vec_fdb_ao_out_b_name="vaob",
    pto_vec_fdb_ao_in_s_sig_name="fdbs",
    pto_vec_fdb_ao_in_s_win_end_sig_name="zlcb",
    pto_vec_fdb_ao_in_b_sig_name="fdbb",
    pto_vec_fdb_ao_in_b_win_end_sig_name="zlcs",
    pto_vec_fdb_ao_in_t_val_name="ao",
    additional_columns_to_drop=None,
    selected_columns_to_keep=None,
    save_outputs=True,
    only_if_target_exist_n_not_zero=True,
    use_fresh=True,
    keep_fdb_count_separated_columns=False,
    keep_bid_ask=True,
    regenerate_cds=False,
    gator_oscillator_flag=False,
    mfi_flag=True,
    balligator_flag=False,
    balligator_period_jaws=89,
    largest_fractal_period=89,
    talligator_flag=False,
    talligator_period_jaws=377,
    use_ttf=True,
):
    if tlid_tag is None:
        tlid_tag = tlid.get_minutes()

    ifn = i.replace("/", "-")
    # read instrument prop
    iprop = pds.get_instrument_properties(i)
    pipsize = iprop["pipsize"]
    nb_decimal = (
        len(str(pipsize).split(".")[1]) if "." in str(pipsize) else len(str(pipsize))
    )
    rounder = nb_decimal

    # Possible crop of the dataframe to a specific date range

    # Read the source data of already calculated CDS
    
    #@STCIssue Generate if not exist
    cds_full_filename = f"{indir_cds}/{ifn}_{t}.csv"
    if not os.path.exists(cds_full_filename) or regenerate_cds or use_fresh:
        from jgtpy import JGTCDS as cds
        reason = "(dont exist)" if not os.path.exists(cds_full_filename) else "" 
        msg = f"JTC is generating the CDS file{reason}: "
        msg = msg + " regenerated flag active" if regenerate_cds else msg
        msg = msg + " use fresh flag active" if use_fresh else msg

        print(msg)
        use_full = True
        #@STCIssue UPGRADE to use JGTCDSSvc
        from jgtpy import JGTCDSSvc as svc
        svc.get(instrument=i, timeframe=t, use_full=use_full, use_fresh=use_fresh)
        # cds.createFromPDSFileToCDSFile(instrument=i, 
        #                                timeframe=t,
        #                                use_full=use_full,
        #                                use_fresh=use_fresh,keep_bid_ask=keep_bid_ask,
        #                                gator_oscillator_flag=gator_oscillator_flag,
        #                                mfi_flag=mfi_flag,
        #                                balligator_flag=balligator_flag,
        #                                balligator_period_jaws=balligator_period_jaws,
        #                                largest_fractal_period=largest_fractal_period,
        #                                talligator_flag=talligator_flag,
        #                                talligator_period_jaws=talligator_period_jaws)#@STCGoal Use Fresh
    if use_ttf:
        from ptottf import read_ttf_csv
        print("                                 jgtml is using ttf....")
        df_cds_source = read_ttf_csv(i, t, use_full=True)
        #print("JGTML::Debug len of df_cds_source:", len(df_cds_source))
    else:
        df_cds_source = pd.read_csv(
            cds_full_filename, index_col=0, parse_dates=True
        )

    df_result_tmx = calculate_target_variable_min_max(
        df_cds_source, 
        crop_end_dt, 
        crop_start_dt, 
        rounder=rounder, 
        pipsize=pipsize,
        WINDOW_MIN=WINDOW_MIN,
        WINDOW_MAX=WINDOW_MAX,
    )

    if pto_vec_fdb_ao_vector_window_flag:
        df_result_tmx = get_fdb_ao_vector_window(
            df_result_tmx,
            out_s_name=pto_vec_fdb_ao_out_s_name,
            out_b_name=pto_vec_fdb_ao_out_b_name,
            in_s_sig_name=pto_vec_fdb_ao_in_s_sig_name,
            in_s_win_end_sig_name=pto_vec_fdb_ao_in_s_win_end_sig_name,
            in_b_sig_name=pto_vec_fdb_ao_in_b_sig_name,
            in_b_win_end_sig_name=pto_vec_fdb_ao_in_b_win_end_sig_name,
            in_t_val_name=pto_vec_fdb_ao_in_t_val_name,
            only_if_target_exist_n_not_zero=only_if_target_exist_n_not_zero,
            
        )
        #@STCGoal Count of the ao vector in the window
        df_result_tmx.loc[:, VECTOR_AO_FDBS_COUNT] = df_result_tmx[VECTOR_AO_FDBS].apply(lambda x: len(x.split(',')))
        df_result_tmx.loc[:, VECTOR_AO_FDBB_COUNT] = df_result_tmx[VECTOR_AO_FDBB].apply(lambda x: len(x.split(',')))
        
        df_result_tmx.loc[:, VECTOR_AO_FDBS_COUNT] = df_result_tmx.apply(lambda x: 0 if x[VECTOR_AO_FDBS_COUNT] == 1 else x[VECTOR_AO_FDBS_COUNT], axis=1)
        df_result_tmx.loc[:, VECTOR_AO_FDBB_COUNT] = df_result_tmx.apply(lambda x: 0 if x[VECTOR_AO_FDBB_COUNT] == 1 else x[VECTOR_AO_FDBB_COUNT], axis=1)
        
        if keep_fdb_count_separated_columns:                
            sel_1_keeping_columns.append(VECTOR_AO_FDBS_COUNT)
            sel_2_keeping_columns.append(VECTOR_AO_FDBS_COUNT)
            sel_1_keeping_columns.append(VECTOR_AO_FDBB_COUNT)
            sel_2_keeping_columns.append(VECTOR_AO_FDBB_COUNT)
            
        
        #VECTOR_AO_FDB_COUNT
        try:sel_1_keeping_columns.append(VECTOR_AO_FDB_COUNT)
        except:pass
        try:sel_2_keeping_columns.append(VECTOR_AO_FDB_COUNT)
        except:pass
        try:selected_columns_to_keep.append(VECTOR_AO_FDB_COUNT)
        except:pass
        
        df_result_tmx.loc[:, VECTOR_AO_FDB_COUNT] = df_result_tmx[VECTOR_AO_FDBS_COUNT] + df_result_tmx[VECTOR_AO_FDBB_COUNT]
        #VECTOR_AO_FDB_COUNT column gets the value of the vector ao fdbs count or fdbb count if not 1
        #df_result_tmx.loc[:, VECTOR_AO_FDB_COUNT] = df_result_tmx.apply(lambda x: x[VECTOR_AO_FDBS_COUNT] if x[VECTOR_AO_FDBS_COUNT] != 0 else x[VECTOR_AO_FDBB_COUNT] != 0, axis=1)
        
        # df_result_tmx.loc[:, VECTOR_AO_FDB_COUNT] = df_result_tmx.apply(lambda x: x[VECTOR_AO_FDBS_COUNT] if x[VECTOR_AO_FDBS_COUNT] != 1 else x[VECTOR_AO_FDBB_COUNT] != 1, axis=1)
        
        # df_result_tmx.loc[:, VECTOR_AO_FDB_COUNT] = df_result_tmx.apply(lambda x: 0 if x[VECTOR_AO_FDB_COUNT] == 1 else x[VECTOR_AO_FDB_COUNT], axis=1)
        
        # #fill with zero if nan or 1
        df_result_tmx[VECTOR_AO_FDB_COUNT] = df_result_tmx[VECTOR_AO_FDB_COUNT].fillna(0)
        #VECTOR_AO_FDB_COUNT column gets the value of the vector ao fdbs count or fdbb count if not 0
        #df_result_tmx.loc[:, VECTOR_AO_FDB_COUNT] = df_result_tmx.apply(lambda x: x[VECTOR_AO_FDBS_COUNT] if x[VECTOR_AO_FDBS_COUNT] != 0 else x[VECTOR_AO_FDBB_COUNT] != 0, axis=1)
        #df_result_tmx.loc[:, VECTOR_AO_FDB_COUNT] =
        
        #df_result_tmx[VECTOR_AO_FDB_COUNT] = df_result_tmx[VECTOR_AO_FDB_COUNT].fillna(0)
        
        if pto_vec_fdb_ao_out_s_name  not in sel_1_keeping_columns:
            sel_1_keeping_columns.append(pto_vec_fdb_ao_out_s_name )
        
        if pto_vec_fdb_ao_out_b_name not in sel_1_keeping_columns:
            sel_1_keeping_columns.append(pto_vec_fdb_ao_out_b_name )
            
        if pto_vec_fdb_ao_out_s_name  not in sel_2_keeping_columns:
            sel_2_keeping_columns.append(pto_vec_fdb_ao_out_s_name )
        
        if pto_vec_fdb_ao_out_b_name not in sel_2_keeping_columns:
            sel_2_keeping_columns.append(pto_vec_fdb_ao_out_b_name )      
        
        

    """  #@STCIssue We will Want those columns in the output selections
        out_s_name = 'vector_ao_fdbs', # REPLACED 240506 vaos
        out_b_name = 'vector_ao_fdbb', # REPLACED 240506 vaob
        in_s_sig_name = 'fdbs',
        in_s_win_end_sig_name = 'zlcb',            
        in_b_sig_name = 'fdbb',
        in_b_win_end_sig_name = 'zlcs',
        in_t_val_name = 'ao'
    """

    # Selection 1
    sel1 = df_result_tmx[sel_1_keeping_columns].copy()
    
    # Select only rows where profit or loss
    sel1 = sel1[(sel1["p"] != 0) | (sel1["l"] != 0)]


    if save_outputs:
        print("Saving MX Target data to file...")
        output_sel_cols_fn = f"{outdir_tmx}/{ifn}_{t}{sel_1_suffix}.csv"
        try:
            sel1.to_csv(output_sel_cols_fn, index=True)
            print(f"Saved to {output_sel_cols_fn}")
        except Exception as e:
            print(f"Error occurred while saving to {output_sel_cols_fn}: {str(e)}")
    
    sel2 = df_result_tmx[sel_2_keeping_columns].copy()
    sel2["target"] = sel2["target"].round(rounder)
    sel2 = sel2[(sel2["target"] != 0)]

    output_tnd_targetNdata_fn = f"{outdir_tmx}/{ifn}_{t}{sel_2_suffix}.csv"
    
    if save_outputs:
        try:
            sel2.to_csv(output_tnd_targetNdata_fn, index=True)
            print(f"Saved to {output_tnd_targetNdata_fn}")
        except Exception as e:
            print(f"Error occurred while saving to {output_tnd_targetNdata_fn}: {str(e)}")

    if selected_columns_to_keep is not None:
        print("   Selected columns to keep:", selected_columns_to_keep)

    if drop_calc_col and selected_columns_to_keep is None:
        print("Dropping calculated columns:", calc_col_to_drop_names)
        df_result_tmx.drop(columns=calc_col_to_drop_names, inplace=True)
    
    
        
    if additional_columns_to_drop is not None and selected_columns_to_keep is not None:
        print("You cannot use both additional_columns_to_drop and selected_columns_to_keep. Please use only one of them.")
        return df_result_tmx, sel1, sel2
    
    if additional_columns_to_drop is not None:
        #print("Dropping additional columns:", additional_columns_to_drop)
        try:
            df_result_tmx.drop(columns=additional_columns_to_drop, inplace=True)
        except:
            pass
    
    if selected_columns_to_keep is not None:
        try:
            df_result_tmx = df_result_tmx[selected_columns_to_keep].copy()
        except:
            pass
    
    if save_outputs:
        # Save the result to a csv file
        output_all_cols_fn = f"{outdir_tmx}/{ifn}_{t}.csv"
        try:
            df_result_tmx.to_csv(output_all_cols_fn, index=True)
            print(f"Saved to {output_all_cols_fn}")
        except Exception as e:
            print(f"Error occurred while saving to {output_all_cols_fn}: {str(e)}")

    if write_reporting:
        _reporting(sel2, ifn, t, pipsize, tlid_tag, output_report_dir=output_report_dir)

    return df_result_tmx, sel1, sel2


def _reporting(df_selection2, ifn, t, pipsize, tlid_tag, output_report_dir=None):

    if output_report_dir is None:
        output_report_dir = os.path.join(os.getenv("JGTPY_DATA_FULL"),"reports")
    if not os.path.exists(output_report_dir):
        os.makedirs(output_report_dir, exist_ok=True)

    report_file = f"{output_report_dir}/report-calc-{tlid_tag}.txt"
    reporting_flag=True if os.getenv("JGT_REPORTING_FLAG") == "True" else False
    if reporting_flag:
        print("Reporting to:", report_file)
        print(" tail -f ", report_file)
        with open(report_file, "a") as f:
            f.write(f"--- {ifn}_{t} --pipsize:{pipsize}---\n")
            f.write(f"Sum of target: {df_selection2['target'].sum()}\n\n")
            # f.write(f" (rounded): {(df_selection2['target'].sum()).round(2)}\n\n")


def readMXFile(
    instrument,
    timeframe,
    columns_to_remove=None,
    quiet=True,
    use_full=True,
    crop_last_dt=None,
    quote_count=None,
    sel_1_suffix="_sel",
    sel_2_suffix="_tnd",
    also_read_selections=False,
    generate_if_not_exist=True,
):
    """
    Read a MX Target file and return a pandas DataFrame.

    Parameters:
    instrument (str): The instrument name.
    timeframe (str): The timeframe of the data.
    columns_to_remove (list, optional): List of column names to remove from the DataFrame. Default is None.
    quiet (bool, optional): If True, suppresses the output messages. Default is True.
    use_full (bool, optional): If True, reads the full MX file. Default is True (there wont be MX in current data I think)).
    crop_last_dt  (str, optional): The last date to crop the data. Default is None.
    quote_count (int, optional): The number of quotes to keep. Default is None.
    sel_1_suffix (str, optional): The suffix for the first selection. Defaults to "_sel".
    sel_2_suffix (str, optional): The suffix for the second selection. Defaults to "_tnd".
    also_read_selections (bool, optional): If True, also read the selections. Defaults to False.
    generate_if_not_exist (bool, optional): If True, generate the MX Target data if it does not exist. Default is True.

    Returns:
    pandas.DataFrame: The DataFrame containing the MX Target data.
    OR
    tuple: A tuple containing the DataFrame and the selections DataFrames.
    """
    # Define the file path based on the environment variable or local path
    data_path_cds = get_data_path("targets/mx", use_full=use_full)
    fpath = pds.mk_fullpath(instrument, timeframe, "csv", data_path_cds)
    
    try:
        mdf = pd.read_csv(fpath)
    except:
        print(f"Error reading file {fpath}")
        print("GENERATING THE MX Targets")
        try:
            pto_target_calculation(instrument,timeframe,pto_vec_fdb_ao_vector_window_flag=True,
                drop_calc_col=False,
                selected_columns_to_keep=ML_DEFAULT_COLUMNS_TO_KEEP)
        except:
            raise ValueError(f"Error generating file {fpath}")
        try:
            mdf = pd.read_csv(fpath)
        except:
            raise ValueError(f"Error reading file {fpath}")
    

    # Set 'Date' as the index and convert it to datetime
    mdf["Date"] = pd.to_datetime(mdf["Date"])
    mdf.set_index("Date", inplace=True)
    # Remove the specified columns
    if columns_to_remove is not None:
        mdf = mdf.drop(columns=columns_to_remove, errors="ignore")

    if crop_last_dt is not None:
        mdf = mdf[mdf.index <= crop_last_dt]
    if quote_count is not None:
        mdf = mdf[-quote_count:]

    if also_read_selections:
        sel1 = pd.read_csv(
            fpath.replace(".csv", f"{sel_1_suffix}.csv"), index_col=0, parse_dates=True
        )
        sel2 = pd.read_csv(
            fpath.replace(".csv", f"{sel_2_suffix}.csv"), index_col=0, parse_dates=True
        )
        return mdf, sel1, sel2
    return mdf




def get_fdb_ao_vector_window(
    df,
    out_s_name="vaos",#VECTOR_AO_FDBS_COUNT
    out_b_name="vaob",#VECTOR_AO_FDBB
    in_s_sig_name="fdbs",
    in_s_win_end_sig_name="zlcb",
    in_b_sig_name="fdbb",
    in_b_win_end_sig_name="zlcs",
    in_t_val_name="ao",
    only_if_target_exist_n_not_zero=True,
):

    # reset index before we iterate
    try:
        df.reset_index(drop=False, inplace=True)
    except:
        pass

    df[out_s_name] = np.nan
    df[out_b_name] = np.nan

    col_target_exist_in_dataset = "target" in df.columns
    
    
    for index, row in df.iterrows():
        process_row = True
        if only_if_target_exist_n_not_zero:
            col_target_exist_in_dataset and row["target"] != 0
            
        if row[in_s_sig_name] == 1 and row[in_t_val_name] > 0 and process_row:
            window_start = index
            window_end = None
            for i in range(index, -1, -1):
                if df.at[i, in_s_win_end_sig_name] == 1:
                    window_end = i
                    break
            window = (
                df.loc[window_end:window_start, in_t_val_name]
                if window_end is not None
                else df.loc[:window_start, in_t_val_name]
            )
            df.at[index, out_s_name] = str(window.astype(float).tolist())
            
        if row[in_b_sig_name] == 1 and row[in_t_val_name] < 0 and process_row:
            window_start = index
            window_end = None
            for i in range(index, -1, -1):
                if df.at[i, in_b_win_end_sig_name] == 1:
                    window_end = i
                    break
            window = (
                df.loc[window_end:window_start, in_t_val_name]
                if window_end is not None
                else df.loc[:window_start, in_t_val_name]
            )
            df.at[index, out_b_name] = str(window.astype(float).tolist())
    # restore index
    try:
        df.set_index("Date", inplace=True)
    except:
        pass

    # for nan values, fill with empty string corresponding to an empty array
    df[out_s_name].fillna("[]", inplace=True)
    df[out_b_name].fillna("[]",inplace=True)

    return df



#@STCGoal Features the Fractals in Relationship to the AO



def get_fdb_ao_vector_window_v2(
    df,
    out_s_name="vaos",
    out_b_name="vaob",
    in_s_sig_name="fdbs",
    in_s_win_end_sig_name="zlcb",
    in_b_sig_name="fdbb",
    in_b_win_end_sig_name="zlcs",
    in_t_val_name="ao",
    only_if_target_exist_n_not_zero=True,
    fractal_count_pto=True,
    fractal_count_col_name="f_aow_cnt", #@STCIssue What do we want to know ?
):

    # reset index before we iterate
    try:
        df.reset_index(drop=False, inplace=True)
    except:
        pass

    df[out_s_name] = np.nan
    df[out_b_name] = np.nan

    col_target_exist_in_dataset = "target" in df.columns
    
    
    for index, row in df.iterrows():
        process_row = True
        if only_if_target_exist_n_not_zero:
            col_target_exist_in_dataset and row["target"] != 0
            
        if row[in_s_sig_name] == 1 and row[in_t_val_name] > 0 and process_row:
            window_start = index
            window_end = None
            for i in range(index, -1, -1):
                if df.at[i, in_s_win_end_sig_name] == 1:
                    window_end = i
                    break
            window = (
                df.loc[window_end:window_start, in_t_val_name]
                if window_end is not None
                else df.loc[:window_start, in_t_val_name]
            )
            df.at[index, out_s_name] = str(window.astype(float).tolist())
            
        if row[in_b_sig_name] == 1 and row[in_t_val_name] < 0 and process_row:
            window_start = index
            window_end = None
            for i in range(index, -1, -1):
                if df.at[i, in_b_win_end_sig_name] == 1:
                    window_end = i
                    break
            window = (
                df.loc[window_end:window_start, in_t_val_name]
                if window_end is not None
                else df.loc[:window_start, in_t_val_name]
            )
            df.at[index, out_b_name] = str(window.astype(float).tolist())
    # restore index
    try:
        df.set_index("Date", inplace=True)
    except:
        pass

    # for nan values, fill with empty string corresponding to an empty array
    df[out_s_name].fillna("[]", inplace=True)
    df[out_b_name].fillna("[]",inplace=True)

    return df







import pandas as pd
from jgtpy import JGTCDSSvc as svc
from jgtutils import jgtpov as jpov
from jgtutils.jgtconstants import (MFI_VAL, ZCOL, AO)

import os

def get_ttf_basedir(use_full,ns="ttf"):
    if use_full:
        bd=os.getenv("JGTPY_DATA_FULL")
        if bd is None:
            raise Exception("JGTPY_DATA_FULL environment variable is not set.")
    else:
        bd=os.getenv("JGTPY_DATA")
        if bd is None:
            raise Exception("JGTPY_DATA environment variable is not set.")
    fulldir=os.path.join(bd,ns)
    #mkdir -p fulldir
    os.makedirs(fulldir, exist_ok=True)
    return fulldir

def get_ttf_outfile_fullpath(i,t,use_full,suffix="",ns="ttf"):
    save_basedir=get_ttf_basedir(use_full,ns)
    ifn=i.replace("/","-")
    output_filename = f"{ifn}_{t}_ttf{suffix}.csv"
    return os.path.join(save_basedir,output_filename)
  
columns_to_get_from_higher_tf = [MFI_VAL, ZCOL, AO]

def make_htf_created_columns_array(workset,t):
    created_columns=[]
    for c in columns_to_get_from_higher_tf:
      for k in workset:
        if not c in created_columns: 
          created_columns.append(c)
        new_col_name = c+"_"+k
        if k != t:
          if not new_col_name in created_columns: 
            created_columns.append(new_col_name)
    return created_columns

def read_ttf_csv(i, t, use_full=False):
    output_filename=get_ttf_outfile_fullpath(i,t,use_full)
    if not os.path.exists(output_filename):
        print("   Non existent, Creating TTF: ", output_filename)
        return create_ttf_csv(i, t, use_full,force_read=True)
    else:
        print("   Read TTF: ", output_filename)
        return pd.read_csv(output_filename, index_col=0)
  
def read_ttf_csv_selection(i, t, use_full=False):
    output_filename_sel=get_ttf_outfile_fullpath(i,t,use_full,suffix="_sel")
    return pd.read_csv(output_filename_sel, index_col=0)
 
def create_ttf_csv(i, t, use_full=False, use_fresh=True, quotescount=-1,force_read=False,dropna=True):
  #if use_full:
  #  print("Using full dataset")

  povs = jpov.get_higher_tf_array(t)
  ttf = pd.DataFrame()

  workset = svc.get_higher_cdf_datasets(i, t, use_full=use_full, use_fresh=use_fresh, quotescount=quotescount, quiet=True, force_read=force_read)
  ttf=workset[t]
  created_columns = make_htf_created_columns_array(workset, t)
  

  for k in workset:  
    if k!=t:
      v=workset[k]
      for c in columns_to_get_from_higher_tf:
      
        new_col_name = c+"_"+k
        ttf[new_col_name]=None

        for ii, row in ttf.iterrows():
          #get the date of the current row (the index)
          date = ii
          #print(k)
          data = v[v.index <= date]
          if not data.empty:
            data = data.iloc[-1]
            ttf.at[ii,new_col_name]=data[c]
  
  columns_we_want_to_keep_to_view=created_columns
  ttf_sel=ttf[columns_we_want_to_keep_to_view].copy()
  
  #save basedir is $JGTPY_DATA/ttf is not use_full, if use_full save basedir is $JGTPY_DATA_FULL/ttf
  
  output_filename=get_ttf_outfile_fullpath(i,t,use_full)
  output_filename_sel=get_ttf_outfile_fullpath(i,t,use_full,suffix="_sel")
  
  if dropna:
    ttf.dropna(inplace=True)
  ttf.to_csv(output_filename, index=True)
  ttf_sel.to_csv(output_filename_sel, index=True)
  print(f"    MX Output full:'{output_filename}'")
  print(f"    MX Output sel :'{output_filename_sel}'")
  return ttf



# This file is generated by pto-pre-build.sh. Do not edit it manually.
# Mon 24 Jun 2024 04:06:51 AM EDT
# SOURCE NAME: jgtml_cli-ttf-proto-01-2406181221.py
# ------------------------------------
import argparse
import pandas as pd
#from jgtpy import JGTCDSSvc as svc
#from jgtutils import jgtpov as jpov
#from jgtutils.jgtconstants import (MFI_VAL, ZCOL, AO)

from jgtml.ptottf import create_ttf_csv

import os

def main():
  parser = argparse.ArgumentParser(description="Create ttf CSV file")
  parser.add_argument("-i", "--instrument", required=True, help="Instrument name")
  parser.add_argument("-t", "--timeframe", required=True, help="Timeframe (e.g., D1, H4)")
  parser.add_argument("-uf", "--full", action="store_true", help="Use full dataset")
  parser.add_argument("-new", "--fresh", action="store_true", help="Use fresh data")
  parser.add_argument("-fr", "--force_read", action="store_true", help="Force to read CDS (should increase speed but relies on existing data)")
  parser.add_argument("-c", "--quotescount", type=int, default=-1, help="Number of quotes to retrieve (default: 333)")

  args = parser.parse_args()
  create_ttf_csv(args.instrument, args.timeframe, args.full if args.full else False, args.fresh, args.quotescount, args.force_read)

if __name__ == "__main__":
  main()
  